import axios from 'axios'
import toCamel from 'camelcase-keys'
import React, { useEffect, useState } from 'react'
import { Helmet } from 'react-helmet'
import { Link, useLocation } from 'react-router-dom'

import TagSidebar from '@/components/TagSidebar'
import WikiBody from '@/components/WikiBody'
import TabGroup, { Tab } from '@/components/common/TabGroup'
import MainArea from '@/components/layout/MainArea'
import { API_BASE_URL, SITE_TITLE } from '@/config'

import type { Post, Tag, WikiPage } from '@/types'


export default () => {
  const [posts, setPosts] = useState<Post[] | null> (null)
  const [wikiPage, setWikiPage] = useState<WikiPage | null> (null)

  const location = useLocation ()
  const query = new URLSearchParams (location.search)
  const tagsQuery = query.get ('tags') ?? ''
  const anyFlg = query.get ('match') === 'any'
  const tags = tagsQuery.split (' ').filter (e => e !== '')

  useEffect(() => {
    void (axios.get (`${ API_BASE_URL }/posts`, {
              params: { tags:   tags.join (','),
                        match:  (anyFlg ? 'any' : 'all') } })
          .then (res => setPosts (toCamel (res.data, { deep: true })))
          .catch (err => {
            console.error ('Failed to fetch posts:', err)
            setPosts (null)
          }))

    setWikiPage (null)
    if (tags.length === 1)
      {
        void (axios.get (`${ API_BASE_URL }/wiki/title/${ encodeURIComponent (tags[0]) }`)
              .then (res => setWikiPage (toCamel (res.data, { deep: true })))
              .catch (() => {
                ;
              }))
      }
  }, [location.search])

  return (
      <>
        <Helmet>
          <title>
            {tags.length
             ? `${ tags.join (anyFlg ? ' or ' : ' and ') } | ${ SITE_TITLE }`
             : `${ SITE_TITLE }　〜 ぼざクリも、ぼざろ外も、外伝もあるんだよ`}
          </title>
        </Helmet>
        <TagSidebar posts={posts || []} />
        <MainArea>
          <TabGroup key={wikiPage}>
            <Tab name="広場">
              {posts == null ? 'Loading...' : (
                  posts.length
                  ? (
                     <div className="flex flex-wrap gap-4 p-4">
                       {posts.map (post => (
                           <Link to={`/posts/${ post.id }`}
                                 key={post.id}
                                 className="w-40 h-40 overflow-hidden rounded-lg shadow-md hover:shadow-lg">
                             <img src={post.thumbnail ?? post.thumbnailBase}
                                  className="object-none w-full h-full" />
                           </Link>))}
                     </div>)
                  : '広場には何もありませんよ．')}
            </Tab>
            {(wikiPage && wikiPage.body) && (
                <Tab name="Wiki" init={!(posts?.length)}>
                  <WikiBody body={wikiPage.body} />
                  <div className="my-2">
                    <Link to={`/wiki/${ wikiPage.title }`}>Wiki を見る</Link>
                  </div>
                </Tab>)}
          </TabGroup>
        </MainArea>
      </>)
}
